"use strict";
var __assign = (this && this.__assign) || Object.assign || function(t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
        s = arguments[i];
        for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
            t[p] = s[p];
    }
    return t;
};
var transition_matcher_1 = require("./transition-matcher");
var TransitionSet = (function () {
    function TransitionSet() {
        this.transitions = [];
    }
    TransitionSet.prototype.addTransition = function (transition) {
        if (this.hasTransition(transition)) {
            this.removeTransition(transition);
        }
        this.transitions.push(transition);
    };
    TransitionSet.prototype.addTransitions = function (transitions) {
        var _this = this;
        transitions.forEach(function (transition) { return _this.addTransition(transition); });
    };
    TransitionSet.prototype.addEvent = function (event) {
        var _this = this;
        if (event.transitions.length === 0) {
            throw new Error("Event \"" + event + "\" must have at least one transition");
        }
        event.transitions.forEach(function (transition) {
            _this.addTransition(__assign({}, transition, { event: event.name }));
        });
    };
    TransitionSet.prototype.addEvents = function (events) {
        var _this = this;
        events.forEach(function (event) { return _this.addEvent(event); });
    };
    TransitionSet.prototype.removeTransition = function (transition) {
        var index = this.findIndex(transition);
        if (index === -1) {
            return;
        }
        this.transitions.splice(index, 1);
    };
    TransitionSet.prototype.removeTransitions = function (transitions) {
        var _this = this;
        transitions.forEach(function (transition) { return _this.removeTransition(transition); });
    };
    TransitionSet.prototype.removeEvent = function (eventName) {
        var index = this.transitions.length;
        while (index--) {
            if (this.transitions[index].event !== eventName) {
                continue;
            }
            this.transitions.splice(index, 1);
        }
    };
    TransitionSet.prototype.removeEvents = function (eventNames) {
        var _this = this;
        eventNames.forEach(function (eventName) { return _this.removeEvent(eventName); });
    };
    TransitionSet.prototype.hasTransition = function (predicate) {
        return this.filterTransitions(predicate).length > 0;
    };
    TransitionSet.prototype.hasEvent = function (eventName) {
        return this.filterTransitions({ event: eventName }).length > 0;
    };
    TransitionSet.prototype.filterTransitions = function (predicate) {
        return this.transitions.filter(function (transition) { return transition_matcher_1.matchTransition(transition, predicate); });
    };
    TransitionSet.prototype.findIndex = function (predicate) {
        for (var index = 0; index < this.transitions.length; index++) {
            var transition = this.transitions[index];
            if (transition_matcher_1.matchTransition(transition, predicate)) {
                return index;
            }
        }
        return -1;
    };
    return TransitionSet;
}());
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = TransitionSet;
//# sourceMappingURL=transition-set.js.map