"use strict";
var Transitioner = (function () {
    function Transitioner(transitions) {
        this.transitions = transitions;
    }
    Transitioner.prototype.canTransition = function (toState, fromState) {
        var transitions = this.transitions.filterTransitions({
            from: fromState,
            to: toState,
        });
        return transitions.some(function (transition) {
            if (transition.condition) {
                return transition.condition() !== false;
            }
            return true;
        });
    };
    Transitioner.prototype.transition = function (toState, fromState, callback) {
        if (!this.transitions.hasTransition({ from: fromState })) {
            throw new Error("\"From\" state not found: " + fromState);
        }
        if (!this.transitions.hasTransition({ to: toState })) {
            throw new Error("\"To\" state not found: " + toState);
        }
        if (!this.canTransition(toState, fromState)) {
            return;
        }
        callback({
            from: fromState,
            to: toState,
        });
    };
    Transitioner.prototype.transitionByEvent = function (event, fromState, callback) {
        if (!this.transitions.hasTransition({ from: fromState })) {
            throw new Error("\"From\" state not found: " + fromState);
        }
        if (!this.transitions.hasEvent(event)) {
            throw new Error("Event not found: " + event);
        }
        var transitions = this.transitions.filterTransitions({
            event: event,
            from: fromState,
        });
        for (var index = 0; index < transitions.length; index++) {
            var transition = transitions[index];
            if (this.canTransition(transition.to, fromState)) {
                return callback({
                    from: fromState,
                    to: transition.to,
                });
            }
        }
    };
    return Transitioner;
}());
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = Transitioner;
//# sourceMappingURL=transitioner.js.map